/***********************************************************************
 * Copyright (c) 2003 University of Kent.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.cs.kent.ac.uk/projects/ocl/CPL.html
 ***********************************************************************/

package uk.ac.kent.cs.ocl20;

import java.io.PrintWriter;
import java.io.Reader;
import java.util.List;

import uk.ac.kent.cs.kmf.util.ILog;
import uk.ac.kent.cs.ocl20.semantics.analyser.OclSemanticAnalyser;
import uk.ac.kent.cs.ocl20.semantics.bridge.BridgeFactory;
import uk.ac.kent.cs.ocl20.semantics.bridge.Environment;
import uk.ac.kent.cs.ocl20.semantics.model.types.TypeFactory;
import uk.ac.kent.cs.ocl20.standard.lib.StdLibAdapter;
import uk.ac.kent.cs.ocl20.syntax.ast.contexts.PackageDeclarationAS;
import uk.ac.kent.cs.ocl20.syntax.parser.OclParser;
import uk.ac.kent.cs.ocl20.synthesis.ModelImplementationAdapter;
import uk.ac.kent.cs.ocl20.synthesis.OclCodeGenerator;
import uk.ac.kent.cs.ocl20.synthesis.OclEvaluator;
import uk.ac.kent.cs.ocl20.synthesis.RuntimeEnvironment;

/**
 * @author dha
 *
 */
public interface OclProcessor {
		public Boolean getDebug();
		public void setDebug(Boolean d);
		public ILog getLog();
		public void setLog(ILog log);
	
		public StdLibAdapter getStdLibAdapter();
		public String getStdLibAdapterName();
		
		public ModelImplementationAdapter getModelImplAdapter();
		public String getModelImplAdapterName();

		public BridgeFactory getBridgeFactory();
		public TypeFactory getTypeFactory();
	
		public OclParser getParser();
		public OclSemanticAnalyser getAnalyser();
		public OclEvaluator getEvaluator();
		public OclCodeGenerator getGenerator();
	
		public java.util.Set getModels();
		public void setModels(java.util.Set models);

	
		/**
		 * Parse the passed string as a package declaration.
		 * 
		 * A non null valued result implies a successful (error free) parse.
		 * 
		 */
		public PackageDeclarationAS parse(String str);
		public PackageDeclarationAS parse(String str, ILog log);
		public PackageDeclarationAS parse(String str, ILog log, boolean debug);
		public PackageDeclarationAS parse(Reader input);
		public PackageDeclarationAS parse(Reader input, ILog log);

		/**
		 * Semantic analysis of the passed package declaration from ast
		 * with output going to the System.out.
		 * 
		 * The result contains a list of ContextDeclaration.
		 */
		public List analyse(PackageDeclarationAS pd);
		public List analyse(PackageDeclarationAS pd, ILog log);
		public List analyse(PackageDeclarationAS pd, Environment env, ILog log);

		/**
		 * Semantic analysis of the passed string which contains a package declaration
		 * with output going to the passed log object.
		 * 
		 * The string may be a number of contexts.
		 * The result contains a list of ContextDeclaration.
		 */
		public List analyse(String str);
		public List analyse(String str, ILog log);
		public List analyse(String str, Environment env, ILog log);

		/**
		 * Semantic analysis of the passed string which contains a package declaration
		 * with output going to the passed log object.
		 * 
		 * The string may be a number of contexts.
		 * The result contains a list of ContextDeclaration.
		 */
		public List analyse(Reader reader);
		public List analyse(Reader reader, ILog log);
		public List analyse(Reader reader, Environment env, ILog log);

		/**
		 * Evaluate the passed string as an ocl expression.
		 * (Assumes a single context)
		 * 
		 * If the expression does not start with the string 'context'
		 * it is assumed that no context has been provided and a default
		 * 'context VoidType inv:' is added to the front of the passed expression,
		 * the type 'VoidType' is added the the type environment,
		 * and the OCL value 'Undefined' is added as 'self'.
		 * 
		 * The result contains the value of the evaluated expression.
		 * 
		 */
		public List evaluate(String str);
		public List evaluate(String str, Object self);
		public List evaluate(String str, Object self, ILog log);
		public List evaluate(String str, Environment env, RuntimeEnvironment renv, ILog log);
		
		public List evaluate(Reader reader, Object self, ILog log);
		public List evaluate(Reader reader, Object self);
		public List evaluate(Reader reader, Environment env, RuntimeEnvironment renv, ILog log);

		/**
		 * Generate code for an input passed as a string 
		 * with provided indentation
		 * 
		 * The result contains the list of Pairs.
		 * 
		 */
		public List generate(String str, String indent);
		public List generate(String str, String indent, ILog log);
		public List generate(String str, String indent, Environment env, ILog log);
		
		public List generate(Reader reader, String indent);
		public List generate(Reader reader, String indent, ILog log);
		public List generate(Reader reader, String indent, Environment env, ILog log);
		
		public void generate(Reader input, PrintWriter output, String pkgName, ILog log);

		/**
		 * Create a type environment
		 * that includes the passed Object as the passed name.
		 * ( additional objects can be added using 'setValue("name", object)' )
		 */
		public Environment environment();
		public Environment environment(String name, Object obj);

		/**
		 * Create a runtime environment
		 * that includes the passed Object as the passed name.
		 * ( additional objects can be added using 'setValue("name", object)' )
		 * 
		 */
		public RuntimeEnvironment runtimeEnvironment();
		public RuntimeEnvironment runtimeEnvironment(String name, Object obj);
}
