/***********************************************************************
 * Copyright (c) 2003 University of Kent.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.cs.kent.ac.uk/projects/ocl/CPL.html
 ***********************************************************************/

package uk.ac.kent.cs.ocl20;

import java.io.Reader;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.Vector;

import uk.ac.kent.cs.kmf.util.ILog;
import uk.ac.kent.cs.ocl20.semantics.bridge.Classifier;
import uk.ac.kent.cs.ocl20.semantics.bridge.Environment;
import uk.ac.kent.cs.ocl20.semantics.bridge.Namespace;
import uk.ac.kent.cs.ocl20.semantics.model.contexts.ContextDeclaration;
import uk.ac.kent.cs.ocl20.syntax.ast.contexts.ContextDeclarationAS;
import uk.ac.kent.cs.ocl20.syntax.ast.contexts.PackageDeclarationAS;
import uk.ac.kent.cs.ocl20.synthesis.RuntimeEnvironment;
import uk.ac.kent.cs.ocl20.synthesis.RuntimeEnvironmentImpl;

/**
 * 
 * @author Octavian Patrascoiu
 *
 */

abstract public class OclProcessorImpl
	implements OclProcessor
{
	protected Boolean debug = Boolean.FALSE;
	public void setDebug(Boolean d) {debug = d;}
	public Boolean getDebug() { return debug; }
	
	protected ILog log=null;
	public ILog getLog() { return log; }
	public void setLog(ILog l) { log = l; }
	
	//Can't make impl of TypeFactory, Evaluator, Generator etc common
	// as they all use (directly or indirectly) the BridgeFactory which
	// is bridge specific !

	public OclProcessorImpl(ILog log) {
		this.log = log;
		this.models = new HashSet();
	}
	public OclProcessorImpl(Set models, ILog log) {
		this.log = log;
		this.models = models;
	}
	
	/** 
	 * Initialise the models
	 * */
	protected Set models = new HashSet();
	public Set getModels() {return models;}
	public void setModels(Set models) { this.models = models; }

	/**
	 * Parse the passed string as a package declaration.
	 * 
	 * A null valued result implies a successful (error free) parse.
	 * 
	 */
	public PackageDeclarationAS parse(String str) {
		if (!str.startsWith("context")) str = "context OclVoid inv: "+str;
		return parse(str, getLog());
	}
	
	/**
	 * Parse the passed string as a package declaration
	 * with output going to the passed log object.
	 * 
	 * A null valued result implies a successful (error free) parse.
	 * 
	 */
	public PackageDeclarationAS parse(String str, ILog log) {
		if (!str.startsWith("context")) str = "context OclVoid inv: "+str;
		log.resetErrors();
		PackageDeclarationAS pd = getParser().parse(str, log);
		if (log.hasErrors()) return null;
		return pd;
	}
	public PackageDeclarationAS parse(String str, ILog log, boolean debug) {
		if (!str.startsWith("context")) str = "context OclVoid inv: "+str;
		PackageDeclarationAS pd = this.getParser().parse(str, log, debug);
		if (log.hasErrors()) return null;
		return pd;
	}


	/**
	 * Parse the passed file as a package declaration.
	 * 
	 * A null valued result implies a successful (error free) parse.
	 * 
	 */
	public PackageDeclarationAS parse(Reader input) {
		return parse(input, getLog());
	}
	
	/**
	 * Parse the passed string as an OCL expression
	 * with output going to the passed log object.
	 * 
	 * A null valued result implies a successful (error free) parse.
	 * 
	 */
	public PackageDeclarationAS parse(Reader input, ILog log) {
		PackageDeclarationAS pd = this.getParser().parse(input, log, this.getDebug().booleanValue());
		if (log.hasErrors()) return null;
		return pd;
	}

	/**
	 * Semantic analysis of the passed package declaration from ast
	 * with output going to the System.out.
	 * 
	 * The result contains a list of ContextDeclaration.
	 */
	public List analyse(PackageDeclarationAS pd) {
		return analyse(pd, getLog());
	}
	

	public List analyse(PackageDeclarationAS pd, ILog log) {
		//--- Create an environment ---
		Environment env = this.environment();
		return analyse(pd, env, log);
	}

	public  List analyse(PackageDeclarationAS pd, Environment env, ILog log) {
		if (pd == null) return null;
		//--- For each class context ---
		Iterator i = pd.getContextDecls().iterator();
		List contextDecls = new Vector();
		while (i.hasNext()) {
			ContextDeclarationAS contextDeclAS = (ContextDeclarationAS)i.next();
			//--- Semantic analysis
			ContextDeclaration contextDecl = getAnalyser().analyse(contextDeclAS, env, log, getDebug().booleanValue());
			contextDecls.add(contextDecl);
		}
		if (log.hasErrors()) return null;
		return contextDecls;
	}

	/**
	 * Semantic analysis of the passed string which contains a package declaration
	 * with output going to the passed log object.
	 * 
	 * The string may be a number of contexts.
	 * The result contains a list of ContextDeclaration.
	 */
	public List analyse(String str) {
		return analyse(str, getLog());
	}

	/**
	 * Semantic analysis of the passed string which contains a package declaration
	 * with output going to the passed log object.
	 * 
	 * The string may be a number of contexts.
	 * The result contains a list of ContextDeclaration.
	 */
	public List analyse(String str, ILog log) {
		//--- Parse the input ---
		PackageDeclarationAS pd = parse(str, log);
		return analyse(pd, log);
	}

	/**
	 * Semantic analysis of the passed string which contains a package declaration
	 * with output going to the passed log object.
	 * 
	 * The string may be a number of contexts.
	 * The result contains a list of ContextDeclaration.
	 */
	public List analyse(Reader reader) {
		return analyse(reader, getLog());
	}

	/**
	 * Semantic analysis of the passed string which contains a package declaration
	 * with output going to the passed log object.
	 * 
	 * The string may be a number of contexts.
	 * The result contains a list of the values of the evaluated expression.
	 * 
	 */
	public List analyse(Reader reader, ILog log) {
		//--- Parse the input ---
		PackageDeclarationAS pd = parse(reader, log);
		return analyse(pd, log);
	}
	public List analyse(String str, Environment env, ILog log) {
		PackageDeclarationAS pd = parse(str, log);
		return analyse(pd, env, log);
	}
	public List analyse(Reader reader, Environment env, ILog log) {
		PackageDeclarationAS pd = parse(reader, log);
		return analyse(pd, env, log);
	}

	/**
	 * Evaluate the passed string as an ocl expression.
	 * (Assumes a single context)
	 * 
	 * If the expression does not start with the string 'context'
	 * it is assumed that no context has been provided and a default
	 * 'context OclVoid inv:' is added to the front of the passed expression,
	 * the type 'OclVoid' is added the the type environment,
	 * and the OCL value 'Undefined' is added as 'self'.
	 * 
	 * The result contains the value of the evaluated expression.
	 * 
	 */
	public List evaluate(String str) {
		ILog log = getLog();
		if (!str.startsWith("context")) str = "context OclVoid inv: "+str;
		List cs = analyse(str);
		if (cs == null) return null;
		List result = new Vector();
		Iterator i = cs.iterator();
		while (i.hasNext()) {
			ContextDeclaration decl = (ContextDeclaration)i.next();
			List declRes = this.getEvaluator().evaluate(decl, this.runtimeEnvironment("self", this.getStdLibAdapter().Undefined()), log); 
			result.addAll(declRes);
		}
		return result;
	}
	
	/**
	 * Evaluation of the passed string which contains a package declaration
	 * in the context of self
	 * with output going to System.out.
	 * 
	 * The string may be a number of contexts which will be evaluated using same runtime environment.
	 * The result contains a list of the values of the evaluated expressions.
	 * 
	 */
	public List evaluate(String str, Object self) {
		ILog log = getLog();
		return evaluate(str, self, log);
	}

	/**
	 * Semantic analysis of the passed string which contains a package declaration
	 * in the context of self
	 * with output going to the passed log object.
	 * 
	 * The string may be a number of contexts which will be evaluated using same runtime environment.
	 * The result contains a list of the values of the evaluated expressions.
	 * 
	 */
	public List evaluate(String str, Object self, ILog log) {
		List cs = analyse(str, log);
		if (cs == null) return null;
		List result = new Vector();
		Iterator i = cs.iterator();
		while (i.hasNext()) {
			ContextDeclaration decl = (ContextDeclaration)i.next();
			List declRes = this.getEvaluator().evaluate(decl, this.runtimeEnvironment("self", self), log); 
			result.addAll(declRes);
		}
		return result;
	}

	/**
	 * Semantic analysis of the passed file which contains a package declaration
	 * in the context of self
	 * with output going to the passed log object.
	 * 
	 * The string may be a number of contexts which will be evaluated using same runtime environment.
	 * The result contains a list of the values of the evaluated expressions.
	 * 
	 */
	public List evaluate(Reader reader, Object self, ILog log) {
		List cs = analyse(reader, log);
		if (cs == null) return null;
		List result = new Vector();
		Iterator i = cs.iterator();
		while (i.hasNext()) {
			ContextDeclaration decl = (ContextDeclaration)i.next();
			List declRes = this.getEvaluator().evaluate(decl, this.runtimeEnvironment("self", self), log); 
			result.addAll(declRes);
		}
		return result;
	}

	/**
	 * Semantic analysis of the passed file which contains a package declaration
	 * in the context of self
	 * with output going to System.out.
	 * 
	 * The string may be a number of contexts which will be evaluated using same runtime environment.
	 * The result contains a list of the values of the evaluated expressions.
	 * 
	 */
	public List evaluate(Reader reader, Object self) {
		ILog log = getLog();
		return evaluate(reader, log);
	}
	public List evaluate(String str, Environment env, RuntimeEnvironment renv, ILog log) {
		List cs = analyse(str, env, log);
		if (cs == null) return null;
		List result = new Vector();
		Iterator i = cs.iterator();
		while (i.hasNext()) {
			ContextDeclaration decl = (ContextDeclaration)i.next();
			List declRes = this.getEvaluator().evaluate(decl, renv, log); 
			result.addAll(declRes);
		}
		return result;
	}
	public List evaluate(Reader reader, Environment env, RuntimeEnvironment renv, ILog log) {
		List cs = analyse(reader, env, log);
		if (cs == null) return null;
		List result = new Vector();
		Iterator i = cs.iterator();
		while (i.hasNext()) {
			ContextDeclaration decl = (ContextDeclaration)i.next();
			List declRes = this.getEvaluator().evaluate(decl, renv, log); 
			result.addAll(declRes);
		}
		return result;
	}

	/**
	 * Generate code for an input passed as a string 
	 * with provided indentation
	 * 
	 * The result contains the list of Pairs.
	 * 
	 */
	public List generate(String str, String indent) {
		ILog log = getLog();
		List cs = analyse(str);
		if (cs == null) return null;
		List result = new Vector();
		Iterator i = cs.iterator();
		while (i.hasNext()) {
			ContextDeclaration decl = (ContextDeclaration)i.next();
			List declRes = this.getGenerator().generate(decl, indent, this); 
			result.addAll(declRes);
		}
		return result;
	}

	/**
	 * Generate code for an input passed as a string 
	 * with provided indentation
	 * 
	 * The result contains the list of Pairs.
	 * 
	 */
	public List generate(String str, String indent, ILog log) {
		List cs = analyse(str, log);
		if (cs == null) return null;
		List result = new Vector();
		Iterator i = cs.iterator();
		while (i.hasNext()) {
			ContextDeclaration decl = (ContextDeclaration)i.next();
			List declRes = this.getGenerator().generate(decl, indent, this); 
			result.addAll(declRes);
		}
		return result;
	}

	/**
	 * Generate code for constraints passed as a file 
	 * with provided indentation
	 * and provided log
	 * 
	 * The result contains the name of the variable that contains the value.
	 * 
	 */
	public List generate(Reader reader, String indent) {
		ILog log = getLog();
		List cs = analyse(reader, log);
		if (cs == null) return null;
		List result = new Vector();
		Iterator i = cs.iterator();
		while (i.hasNext()) {
			ContextDeclaration decl = (ContextDeclaration)i.next();
			List declRes = this.getGenerator().generate(decl, indent, this); 
			result.addAll(declRes);
		}
		return result;
	}

	/**
	 * Generate code for constraints passed as a file 
	 * with provided indentation
	 * and provided log
	 * 
	 * The result contains the name of the variable that contains the value.
	 * 
	 */
	public List generate(Reader reader, String indent, ILog log) {
		List cs = analyse(reader, log);
		if (cs == null) return null;
		List result = new Vector();
		Iterator i = cs.iterator();
		while (i.hasNext()) {
			ContextDeclaration decl = (ContextDeclaration)i.next();
			List declRes = this.getGenerator().generate(decl, indent, this); 
			result.addAll(declRes);
		}
		return result;
	}
	
	public List generate(String str, String indent, Environment env, ILog log) {
		List cs = analyse(str, env, log);
		if (cs == null) return null;
		List result = new Vector();
		Iterator i = cs.iterator();
		while (i.hasNext()) {
			ContextDeclaration decl = (ContextDeclaration)i.next();
			List declRes = this.getGenerator().generate(decl, indent, this); 
			result.addAll(declRes);
		}
		return result;
	}
	public List generate(Reader reader, String indent, Environment env, ILog log) {
		List cs = analyse(reader, env, log);
		if (cs == null) return null;
		List result = new Vector();
		Iterator i = cs.iterator();
		while (i.hasNext()) {
			ContextDeclaration decl = (ContextDeclaration)i.next();
			List declRes = this.getGenerator().generate(decl, indent, this); 
			result.addAll(declRes);
		}
		return result;
	}

	/**
	 * Creates an environment for 'self'  
	 * ( additional objects can be added using 'setValue("name", object)' )
	 */
	public Environment environment(String selfName, Object selfType) {
		Classifier bridgeType = getBridgeFactory().buildClassifier(selfType);
		Environment env = this.environment();
		return env.addVariableDeclaration(selfName, bridgeType, Boolean.TRUE);
	}

	public Environment environment() {
		Environment env = getBridgeFactory().buildEnvironment();
		// Add basicTypes;
		env = env.addElement("OclAny", getTypeFactory().buildOclAnyType(), Boolean.FALSE);
		env = env.addElement("OclVoid", getTypeFactory().buildVoidType(), Boolean.FALSE);
		env = env.addElement("Real", getTypeFactory().buildRealType(), Boolean.FALSE);
		env = env.addElement("Integer", getTypeFactory().buildIntegerType(), Boolean.FALSE);
		env = env.addElement("String", getTypeFactory().buildStringType(), Boolean.FALSE);
		env = env.addElement("Boolean", getTypeFactory().buildBooleanType(), Boolean.FALSE);
		Iterator i = getModels().iterator();
		while (i.hasNext()) {
			Namespace ns = getBridgeFactory().buildNamespace(i.next());
			env = env.addNamespace(ns);
		}
		return env;
	}

	/**
	 * Create a runtime environment
	 * that includes the passed EObject as the passed name.
	 * ( additional objects can be added using 'setValue("name", object)' )
	 * 
	 */
	public RuntimeEnvironment runtimeEnvironment(String name, Object eobj) {
		RuntimeEnvironment renv = new RuntimeEnvironmentImpl();
		renv.setValue(name, eobj);
		return renv;
	}
	public RuntimeEnvironment runtimeEnvironment() {
		RuntimeEnvironment renv = new RuntimeEnvironmentImpl();
		return renv;
	}
}
